unit UAproxim;

interface

uses
  SysUtils, Windows, Types;

type

  TSoustava = class
  private
    FPocet: Integer;
  public
    Matice: array of array of Real;
    Reseni: array of Real;
    procedure Rad(R: Integer);
    procedure Vyres;
  end;

  TAproximace = class
  private
    FData: array of record
      X,Y :Real;
    end;
    FPocet: Integer;
  public
    KoeficientyPolynomu: array of Real;
    Rad: Integer;                         (* d vslednho polynomu *)
    procedure Aproximuj;
    procedure Reset(R: Integer);
    procedure Pridej(Y, X: Real);
  end;

  TPoint2 = record
    X0, Y0, X1, Y1 :Real;
  end;

  TAproximace2D = class
  private
    FData: array of TPoint2;
    FPocet: Integer;
    Rad: Integer;
    KIndex: Integer;
    procedure FAproximuj;
    function GetData(Index: Integer): TPoint2;
  public
    Koeficienty: array[0..1,0..5] of Real;
    procedure Aproximuj;
    procedure Reset;
    procedure Pridej(X0, Y0, X1, Y1: Real);
    function Transform(P: TPoint): TPoint;
    property Pocet:Integer read FPocet;
    property Data[Index: Integer]: TPoint2 read GetData;
  end;

implementation

function Mocnina(Z: Real; M: Integer): Real;
var
  ZZ: Real;
  I: Integer;
begin
  ZZ:=1;
  for I:= 1 to M do ZZ:=ZZ*Z;
  Mocnina:= ZZ;
end;

(* == TSoustava ============================================================= *)

procedure TSoustava.Rad(R: Integer);
begin
  SetLength(Matice, R, R+1);
  SetLength(Reseni, R);
  FPocet:= R;
end;

procedure TSoustava.Vyres;
var
  K: Real;
  S, X, Y: Integer;
begin
  // Gausova eliminace
  for S:= 0 to FPocet-1 do begin
    // Vyjednikovn prvnch len
    for Y:= S to FPocet-1 do begin
      K:= 1/(Matice[Y,S]+0.000000001);
      for X:= S to FPocet do begin
        Matice[Y,X]:= Matice[Y,X]*K;
      end;
    end;
    // Odeten zpornho pedchozho dku
    for Y:= S+1 to FPocet-1 do begin
      for X:= S to FPocet do Matice[Y,X]:= Matice[Y,X]-Matice[S,X];
    end;
  end;
  // Jordanova eliminace
  for S:= 0 to FPocet-2 do
  for Y:= 0 to S do begin
    K:= Matice[Y,S+1];
    for X:= S+1 to FPocet do Matice[Y,X]:= Matice[Y,X]-Matice[S+1,X]*K;
  end;
  // Zznam vsledk
  for X:= 0 to FPocet-1 do Reseni[X]:= Matice[X,FPocet];
end;

(* == TAproximace =========================================================== *)

procedure TAproximace.Pridej(Y, X: Real);
begin
  Inc(FPocet);
  SetLength(FData,FPocet);
  FData[FPocet-1].X:= X;
  FData[FPocet-1].Y:= Y;
end;

procedure TAproximace.Reset(R: Integer);
begin
  SetLength(KoeficientyPolynomu,R+1);
  SetLength(FData,0);
  FPocet:= 0;
  Rad:= R;
end;

procedure TAproximace.Aproximuj;
var
  Soustava: TSoustava;
  I, II: Integer;
  S, T: array of Real;
begin
  // Vpoet koeficient s
  SetLength(S,Rad*2+1);
  for I:= 0 to Rad*2 do
  for II:= 0 to FPocet-1 do S[I]:= S[I]+ Mocnina(FData[II].X,I);
  // Vpoet koeficient t
  SetLength(T,Rad+1);
  for I:= 0 to Rad do
  for II:= 0 to FPocet-1 do T[I]:= T[I]+ Mocnina(FData[II].X,I)*FData[II].Y;
  // Naplnn soustavy
  Soustava:= TSoustava.Create;
  try
    Soustava.Rad(Rad+1);
    for I:= 0 to Rad do
    for II:= 0 to Rad do Soustava.Matice[I,II]:= S[II+I];
    for I:= 0 to Rad do Soustava.Matice[I,Rad+1]:= T[I];
    Soustava.Vyres;
    for I:= 0 to Rad do KoeficientyPolynomu[I]:= Soustava.Reseni[I];
  finally
    Soustava.Free;
  end;
end;

(* == TAproximace2D ========================================================= *)

procedure TAproximace2D.Pridej(X0, Y0, X1, Y1: Real);
begin
  Inc(FPocet);
  SetLength(FData,FPocet);
  FData[FPocet-1].X0:= X0;
  FData[FPocet-1].Y0:= Y0;
  FData[FPocet-1].X1:= X1;
  FData[FPocet-1].Y1:= Y1;
end;

procedure TAproximace2D.Reset;
begin
  SetLength(FData,0);
  FPocet:= 0;
  Rad:= 5;
end;

procedure TAproximace2D.FAproximuj;
var
  Soustava: TSoustava;
  I, II, I2: Integer;
  S: array[0..5,0..5] of Real;
  T: array[0..5] of Real;
  Pole: array[0..5] of Real;
begin
  for I:= 0 to Rad do begin
    T[I]:= 0;
    for II:= 0 to Rad do S[I,II]:= 0;
  end;
  for II:= 0 to FPocet-1 do begin
    // Vpoet pomocnho pole
    with FData[II] do begin
      Pole[0]:= 1;
      Pole[1]:= X0;
      Pole[2]:= Y0;
      Pole[3]:= X0*X0;
      Pole[4]:= Y0*Y0;
      Pole[5]:= X0*Y0;
    end;

    // Vpoet koeficient t
    if KIndex = 0 then begin
      for I:= 0 to Rad do T[I]:= T[I]+ Pole[I]*FData[II].X1;
    end else for I:= 0 to Rad do T[I]:= T[I]+ Pole[I]*FData[II].Y1;

    // Vpoet koeficient s
    for I:= 0 to Rad do
      for I2:= 0 to Rad do
        S[I,I2]:= S[I,I2] + Pole[I]*Pole[I2];
  end;

  // Naplnn soustavy
  Soustava:= TSoustava.Create;
  try
    Soustava.Rad(Rad+1);
    for I:= 0 to Rad do
      for II:= 0 to Rad do Soustava.Matice[I,II]:= S[I,II];
    for I:= 0 to Rad do Soustava.Matice[I,Rad+1]:= T[I];
    Soustava.Vyres;
    for I:= 0 to Rad do Koeficienty[KIndex,I]:= Soustava.Reseni[I];
  finally
    Soustava.Free;
  end;
end;

procedure TAproximace2D.Aproximuj;
var
  I: Integer;
begin
  KIndex:= 0;
  FAproximuj;
  KIndex:= 1;
  FAproximuj;
end;

function TAproximace2D.Transform(P: TPoint): TPoint;
begin
  Result:= Point(Round(Koeficienty[0,0]+Koeficienty[0,1]*P.X+Koeficienty[0,2]*P.Y+
  Koeficienty[0,3]*P.X*P.X+Koeficienty[0,4]*P.Y*P.Y+Koeficienty[0,5]*P.X*P.Y),
  Round(Koeficienty[1,0]+Koeficienty[1,1]*P.X+Koeficienty[1,2]*P.Y+
  Koeficienty[1,3]*P.X*P.X+Koeficienty[1,4]*P.Y*P.Y+Koeficienty[1,5]*P.X*P.Y));
end;

function TAproximace2D.GetData(Index: Integer): TPoint2;
begin
  Result:= FData[Index];
end;

end.
