unit UMainWindow;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, DSPack, StrUtils, ComCtrls, UAutoRegistry, ExtCtrls,
  Spin, UAproxim, CommPort;

const
  DIBDataMaxLength = 2000000;
  ConfigFileName = 'Options.cfg';
  FrameWidth = 1023;
  FrameHeight = 767;
  PixelsSize = (FrameWidth+1) * (FrameHeight+1);

type
  TDIBData = array[0..DIBDataMaxLength] of Integer;
  PDIBData = ^TDIBData;
  TMethod = procedure of object;
  TPinType = (tpInteger, tpBoolean, tpReal, tpString, tpTPoint, tpTScanImage, tpTColor);

  TProcessStepType = (pkCompute, pkShow, pkBoth);

  TIntegerType = class
    Value: Integer;
  end;
  TBooleanType = class
    Value: Boolean;
  end;
  TPointType = class
    Value: TPoint;
  end;
  TRectangleType = class
    Rectangle: TRect;
  end;
  TRealType = class
    Value: Real;
  end;
  TColorType = class
    Color: TColor;
  end;

  TPin = record
    Typ: TPinType;
    Data: Pointer;
  end;

  TPinArray = class
    Pins: array of Integer;
    procedure AddPin(Index: Integer);
  end;

  TStep = class
    StepType: Integer;
    Inputs: TPinArray;
    Outputs: TPinArray;
    Properties: array of TPin;
    function StepName: string;
    constructor Create;
  end;

  TPoleBajtu = array of Byte;
  TScanImage = class
    Left: Integer;
    Top: Integer;
    Width: Integer;
    Height: Integer;
    Pixels: array[0..FrameHeight,0..FrameWidth] of Byte;
    procedure SetDimension(X, Y: Integer);
  end;

  TDetekce = class
  private
  public
    Bod: TPoint;
    Aproxim: TAproximace2D;
    PinValues: array of TPin;
    Buffer: Pointer;
    BufferLength: Integer;
    Width: Integer;
    Height: Integer;
    procedure InitPinValues;
    function GetStepPinDataPointer(Index: Integer): Pointer;
    procedure NactiSnimek;
    procedure NactiSnimek2;
    procedure ZobrazSnimek;
    procedure PrumerCtvercu;
    procedure KontrastniPrechody;
    procedure ProhoditSnimky;
    procedure TmaveOkoli;
    procedure TmaveOkoli2;
    procedure TmaveOkoliASM;
    procedure Vyhlazeni;
    procedure JasKontrast;
    procedure ZobrazKriz;
    procedure PolohaBarvy;
    procedure HledaniObrysu;
    procedure InitPins;
    procedure FinalPoint;
    procedure FinalPoint2;
    function GetLocalPropertyPin(Index: Integer): Pointer;
    function GetLocalInputPin(Index: Integer): Pointer;
    procedure SetLocalInputPin(Index: Integer; const Value: Pointer);
    function GetLocalOutputPin(Index: Integer): Pointer;
    procedure SetLocalOutputPin(Index: Integer; const Value: Pointer);
    property LocalInputPins[Index: Integer]: Pointer read GetLocalInputPin write SetLocalInputPin;
    property LocalOutputPins[Index: Integer]: Pointer read GetLocalOutputPin write SetLocalOutputPin;
    property LocalPropertyPins[Index: Integer]: Pointer read GetLocalPropertyPin;
    procedure ZamereniKrizem;
    procedure RozdilSnimku;
    procedure Prahovani;
    procedure Histogram;
    procedure HledaniObjektuRozdilem;
    procedure HledaniBilehoCtverce;
    procedure RozdilSouradnic;
  published
  end;

  TMetoda = class
    Nazev: string;
    Steps: array of TStep;
    procedure PridatKrok(Step: Integer);
  end;

  TPropertyType = class
    PinType: TPinType;
    Text: string;
  end;

  TStepDef = class
    Name: string;
    Call: TMethod;
    InputsType: array of TPinType;
    OutputsType: array of TPinType;
    PropertiesType: array of TPropertyType;
    ProcessStepType: TProcessStepType;
  end;

  TMainWindow = class(TForm)
    FilterGraph1: TFilterGraph;
    VideoWindow1: TVideoWindow;
    Filter1: TFilter;
    SampleGrabber1: TSampleGrabber;
    Label1: TLabel;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Button4: TButton;
    CheckBox1: TCheckBox;
    ListView1: TListView;
    ListView2: TListView;
    Label2: TLabel;
    Label3: TLabel;
    CheckBox2: TCheckBox;
    CheckBox3: TCheckBox;
    Label4: TLabel;
    Button5: TButton;
    CheckBox4: TCheckBox;
    StatusBar1: TStatusBar;
    Timer1: TTimer;
    GroupBox1: TGroupBox;
    Label5: TLabel;
    Label7: TLabel;
    Button6: TButton;
    CheckBox5: TCheckBox;
    SpinEdit1: TSpinEdit;
    Button7: TButton;
    Button8: TButton;
    SpinEdit2: TSpinEdit;
    Button9: TButton;
    Label6: TLabel;
    GroupBox2: TGroupBox;
    ComboBox1: TComboBox;
    CheckBox6: TCheckBox;
    CommPort1: TCommPort;
    CheckBox7: TCheckBox;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure SampleGrabber1Buffer(sender: TObject; SampleTime: Double;
      pBuffer: Pointer; BufferLen: Integer);
    procedure ListView2SelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure CheckBox1Click(Sender: TObject);
    procedure CheckBox2Click(Sender: TObject);
    procedure CheckBox3Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure Button5Click(Sender: TObject);
    procedure CheckBox4Click(Sender: TObject);
    procedure ListView1SelectItem(Sender: TObject; Item: TListItem;
      Selected: Boolean);
    procedure Timer1Timer(Sender: TObject);
    procedure Button6Click(Sender: TObject);
    procedure SpinEdit1Change(Sender: TObject);
    procedure CheckBox5Click(Sender: TObject);
    procedure Button8Click(Sender: TObject);
    procedure Button7Click(Sender: TObject);
    procedure Button9Click(Sender: TObject);
    procedure CheckBox7Click(Sender: TObject);
  private
    { Private declarations }
  public
    Zamek: Boolean;
    Bits: Pointer;
    BInfo: TBitmapInfo;
    Bitmap: PBitmapInfoHeader;
    BitmapInfoSize: DWORD;
    BitmapBitsSize: DWORD;
    Bitmapa: TBitmap;
    Detekce: TDetekce;
    Methods: array of TMetoda;
    AllStepsList: array of TStepDef;
    Registry: TAutoRegistry;
    ZdrojVidea: Integer;
    ZdrojTyp: Boolean;
    ZdrojRezim: Integer;
    Soubor: string;
    ProvadetFiltry: Boolean;
    ZobrazitNahled: Boolean;
    ZvolenaMetoda: Integer;
    VideoEnabled: Boolean;
    PreviewOnTop: Boolean;
    ShowHistogram: Boolean;
    CurrentMethod: Integer;
    CurrentStep: Integer;
    ShowPropertiesWindow: Boolean;

    Prumerovani: array of TPoint;
    NasobekX, NasobekY: Real;
    UkazatelPrumeru: Integer;
    Posun, KalibraceMin, KalibraceMax: TPoint;

    function PridatMetodu(Nazev: string): TMetoda;
    procedure ChangeVideoResolution(X, Y: Integer);
    procedure StopVideo;
    procedure Zobraz;
    function AddStep(Name: string; Call: TMethod; ProcessStepType: TProcessStepType): TStepDef;
    procedure AddProperty(var Step: TStepDef; Text: string; PinType: TPinType);
    procedure AddInputPin(var Step: TStepDef; Text: string; PinType: TPinType);
    procedure AddOutputPin(var Step: TStepDef; Text: string; PinType: TPinType);
    procedure SaveToFile;
    procedure LoadFromFile;
    function PinTypeToStr(PinType: TPinType): string;
    procedure ListView2Reload;
  end;

var
  MainWindow: TMainWindow;

implementation

uses UOptions, UPreview, UEdit, UPropertiesForm, UTransform, UHistogram;

{$R *.dfm}

procedure InvalidGraphic(Str: PResStringRec);
begin
end;

procedure InvalidBitmap;
begin
end;

procedure InitializeBitmapInfoHeader(Bitmap: HBITMAP; var BI: TBitmapInfoHeader;
  Colors: Integer);
var
  DS: TDIBSection;
  Bytes: Integer;
begin
  DS.dsbmih.biSize := 0;
  Bytes := GetObject(Bitmap, SizeOf(DS), @DS);
  if Bytes = 0 then InvalidBitmap
  else if (Bytes >= (sizeof(DS.dsbm) + sizeof(DS.dsbmih))) and
    (DS.dsbmih.biSize >= DWORD(sizeof(DS.dsbmih))) then
    BI := DS.dsbmih
  else
  begin
    FillChar(BI, sizeof(BI), 0);
    with BI, DS.dsbm do
    begin
      biSize := SizeOf(BI);
      biWidth := bmWidth;
      biHeight := bmHeight;
    end;
  end;
  case Colors of
    2: BI.biBitCount := 1;
    3..16:
      begin
        BI.biBitCount := 4;
        BI.biClrUsed := Colors;
      end;
    17..256:
      begin
        BI.biBitCount := 8;
        BI.biClrUsed := Colors;
      end;
  else
    BI.biBitCount := DS.dsbm.bmBitsPixel * DS.dsbm.bmPlanes;
  end;
  BI.biPlanes := 1;
  if BI.biClrImportant > BI.biClrUsed then
    BI.biClrImportant := BI.biClrUsed;
  if BI.biSizeImage = 0 then
    BI.biSizeImage := BytesPerScanLine(BI.biWidth, BI.biBitCount, 32) * Abs(BI.biHeight);
end;

procedure InternalGetDIBSizes(Bitmap: HBITMAP; var InfoHeaderSize: DWORD;
  var ImageSize: DWORD; Colors: Integer);
var
  BI: TBitmapInfoHeader;
begin
  InitializeBitmapInfoHeader(Bitmap, BI, Colors);
  if BI.biBitCount > 8 then
  begin
    InfoHeaderSize := SizeOf(TBitmapInfoHeader);
    if (BI.biCompression and BI_BITFIELDS) <> 0 then
      Inc(InfoHeaderSize, 12);
  end
  else
    if BI.biClrUsed = 0 then
      InfoHeaderSize := SizeOf(TBitmapInfoHeader) +
        SizeOf(TRGBQuad) * (1 shl BI.biBitCount)
    else
      InfoHeaderSize := SizeOf(TBitmapInfoHeader) +
        SizeOf(TRGBQuad) * BI.biClrUsed;
  ImageSize := BI.biSizeImage;
end;

function InternalGetDIB(Bitmap: HBITMAP; Palette: HPALETTE;
  var BitmapInfo; var Bits; Colors: Integer): Boolean;
var
  OldPal: HPALETTE;
  DC: HDC;
begin
  InitializeBitmapInfoHeader(Bitmap, TBitmapInfoHeader(BitmapInfo), Colors);
  OldPal := 0;
  DC := CreateCompatibleDC(0);
  try
    if Palette <> 0 then
    begin
      OldPal := SelectPalette(DC, Palette, False);
      RealizePalette(DC);
    end;
    Result := GetDIBits(DC, Bitmap, 0, TBitmapInfoHeader(BitmapInfo).biHeight, @Bits,
      TBitmapInfo(BitmapInfo), DIB_RGB_COLORS) <> 0;
  finally
    if OldPal <> 0 then SelectPalette(DC, OldPal, False);
    DeleteDC(DC);
  end;
end;

procedure TMainWindow.Button1Click(Sender: TObject);
begin
  Options.ShowModal;
end;

procedure TMainWindow.ChangeVideoResolution(X, Y: Integer);
begin
with Detekce do begin
    Width:= X;
    Height:= Y;
  end;
//  InternalGetDIB(Bitmapa.Handle, 0, Bitmap^, Bits^, 0);
//  BMax:= Trunc(ObrazMaxX*3/4);
//  MaxX:= X div 2;
//  MaxY:= Y div 2;
//  Canvas.Brush.Color:= clBtnFace;
//  Canvas.Rectangle(0,0,Width,Height);
  // Nastav rozmry okna nhledu
  Preview.Width:= X;
  Preview.Height:= Y+20;
end;

procedure TMainWindow.Button2Click(Sender: TObject);
begin
  if FilterGraph1.State=gsUninitialized then Options.StartVideo;
  VideoEnabled:= True;
  FilterGraph1.Play;
end;

procedure TMainWindow.Button3Click(Sender: TObject);
begin
  FilterGraph1.Pause;
end;

procedure TMainWindow.Button4Click(Sender: TObject);
begin
  VideoEnabled:= False;
  while Zamek do Application.ProcessMessages;
  FilterGraph1.Stop;
end;

procedure TMainWindow.FormCreate(Sender: TObject);
var
  Step: TStepDef;
begin
  Registry:= TAutoRegistry.Create('\Software\Moonlight\Detekce polohy zorni oka');
  with Registry do begin
    Include('SourceType',ZdrojTyp,True);
    Include('SourceFilename',Soubor,'MP2001.avi');
    Include('ProcessFiltrs',ProvadetFiltry,True);
    Include('ShowPreview',ZobrazitNahled,False);
    Include('ChoosenMethod',ZvolenaMetoda,0);
    Include('PreviewOnTop',PreviewOnTop,False);
    Include('ShowPropertiesWindow',ShowPropertiesWindow,True);
    Include('ShowHistogram',ShowHistogram,False);
    LoadFromRegistry;
    CheckBox1.Checked:= ZobrazitNahled;
    CheckBox2.Checked:= ProvadetFiltry;
    CheckBox3.Checked:= PreviewOnTop;
    CheckBox4.Checked:= ShowPropertiesWindow;
    CheckBox7.Checked:= ShowHistogram;
    VideoEnabled:= ProvadetFiltry;
  end;

  Detekce:= TDetekce.Create;
  Detekce.Aproxim:= TAproximace2D.Create;
  Bitmapa:= TBitmap.Create;

  with Detekce do begin
    Step:= AddStep('Nati snmek', NactiSnimek, pkCompute);
      AddOutputPin(Step,'Snmek',tpTScanImage);
      AddProperty(Step,'Rychl natn',tpBoolean);

    Step:= AddStep('Zobraz snmek', ZobrazSnimek, pkShow);
      AddInputPin(Step,'Snmek',tpTScanImage);
      AddProperty(Step,'Rychl vykreslovn',tpBoolean);

    Step:= AddStep('Prvn fze - Prmr tverc', PrumerCtvercu, pkCompute);
      AddInputPin(Step,'Snmek',tpTScanImage);
      AddOutputPin(Step,'Snmek',tpTScanImage);
      AddOutputPin(Step,'Souadnice',tpTPoint);
      AddProperty(Step,'Velikost tevrc',tpInteger);
      AddProperty(Step,'Automatick velikost',tpBoolean);

    Step:= AddStep('Prohodit snmky', ProhoditSnimky, pkCompute);
      AddInputPin(Step,'Snmek',tpTScanImage);
      AddOutputPin(Step,'Snmek',tpTScanImage);

    Step:= AddStep('Kontrastn pechody', KontrastniPrechody, pkCompute);
      AddInputPin(Step,'Snmek',tpTScanImage);
      AddOutputPin(Step,'Snmek',tpTScanImage);
      AddProperty(Step,'Rozmach',tpInteger);
      AddProperty(Step,'Automatick rozmach',tpBoolean);

    Step:= AddStep('Zmna jasu a kontrastu', JasKontrast, pkCompute);
      AddInputPin(Step,'Snmek',tpTScanImage);
      AddOutputPin(Step,'Snmek',tpTScanImage);
      AddProperty(Step,'Kontrast',tpReal);
      AddProperty(Step,'Jas',tpInteger);
      AddProperty(Step,'Automaticky',tpBoolean);

    Step:= AddStep('Prvn fze - Tmav okol 1', TmaveOkoli, pkCompute);
      AddInputPin(Step,'Snmek',tpTScanImage);
      AddOutputPin(Step,'Snmek',tpTScanImage);
      AddOutputPin(Step,'Souadnice',tpTPoint);
      AddProperty(Step,'Rozmach',tpInteger);

    Step:= AddStep('Prvn fze - Tmav okol 2', TmaveOkoli2, pkCompute);
      AddInputPin(Step,'Snmek',tpTScanImage);
      AddOutputPin(Step,'Snmek',tpTScanImage);
      AddOutputPin(Step,'Souadnice',tpTPoint);
      AddProperty(Step,'Rozmach',tpInteger);
      AddProperty(Step,'Automatick rozmach',tpBoolean);

    Step:= AddStep('Zobrazen ke', ZobrazKriz, pkShow);
      AddInputPin(Step,'Souadnice',tpTPoint);
      AddProperty(Step,'Barva',tpTColor);

    Step:= AddStep('Vyhlazen', Vyhlazeni, pkCompute);
      AddInputPin(Step,'Snmek',tpTScanImage);
      AddOutputPin(Step,'Snmek',tpTScanImage);

    Step:= AddStep('Nalezen polohy barvy', PolohaBarvy, pkCompute);
      AddInputPin(Step,'Snmek',tpTScanImage);
      AddOutputPin(Step,'Souadnice',tpTPoint);
      AddProperty(Step,'Hledan barva',tpInteger);

    Step:= AddStep('Hledn obrysu', HledaniObrysu, pkCompute);
      AddInputPin(Step,'Snmek',tpTScanImage);
      AddInputPin(Step,'Souadnice',tpTPoint);
      AddOutputPin(Step,'Snmek',tpTScanImage);
      AddOutputPin(Step,'Souadnice',tpTPoint);
      AddProperty(Step,'Tolerance',tpInteger);

    Step:= AddStep('Histogram', Histogram, pkShow);
      AddInputPin(Step,'Snmek',tpTScanImage);

    Step:= AddStep('Pout zjitn bod', FinalPoint, pkCompute);
      AddInputPin(Step,'Souadnice',tpTPoint);
//      AddProperty(Step,'Ovldat my',tpBoolean);
//      AddProperty(Step,'Poet prmr',tpInteger);

    Step:= AddStep('Prvn fze - tmav okol ASM', TmaveOkoliASM, pkCompute);
      AddInputPin(Step,'Snmek',tpTScanImage);
      AddOutputPin(Step,'Snmek',tpTScanImage);
      AddOutputPin(Step,'Souadnice',tpTPoint);
      AddProperty(Step,'Rozmach',tpInteger);
      AddProperty(Step,'Automatick rozmach',tpBoolean);

    Step:= AddStep('Pout zjitn bod 2', FinalPoint2, pkCompute);
      AddInputPin(Step,'Souadnice',tpTPoint);

    Step:= AddStep('Zamen kem', ZamereniKrizem, pkCompute);
      AddInputPin(Step,'Snmek',tpTScanImage);
      AddInputPin(Step,'Souadnice',tpTPoint);
      AddOutputPin(Step,'Snmek',tpTScanImage);
      AddOutputPin(Step,'Souadnice',tpTPoint);
      AddProperty(Step,'Tolerance',tpInteger);

    Step:= AddStep('Rozdl snmk', RozdilSnimku, pkCompute);
      AddInputPin(Step,'Snmek 1',tpTScanImage);
      AddInputPin(Step,'Snmek 2',tpTScanImage);
      AddOutputPin(Step,'Vsledn snmek',tpTScanImage);

    Step:= AddStep('Prahovn', Prahovani, pkCompute);
      AddInputPin(Step,'Vstupn snmek',tpTScanImage);
      AddOutputPin(Step,'Vstupn snmek',tpTScanImage);
      AddProperty(Step,'Doln rove',tpInteger);
      AddProperty(Step,'Horn rove ',tpInteger);

    Step:= AddStep('Hledn objektu rozdlem', HledaniObjektuRozdilem, pkCompute);
      AddInputPin(Step,'Vstupn snmek',tpTScanImage);
      AddOutputPin(Step,'Vstupn snmek',tpTScanImage);
      AddProperty(Step,'Vnitn rove jasu',tpInteger);
      AddProperty(Step,'Vnj rove jasu',tpInteger);
      AddProperty(Step,'Vnitn polomr',tpInteger);
      AddProperty(Step,'Vnj polomr',tpInteger);

    Step:= AddStep('Hledn blho tverce', HledaniBilehoCtverce, pkCompute);
      AddInputPin(Step,'Vstupn snmek',tpTScanImage);
      AddOutputPin(Step,'Soadnice',tpTPoint);

    Step:= AddStep('Rodl souadnic', RozdilSouradnic, pkCompute);
      AddInputPin(Step,'Souadnice 1',tpTPoint);
      AddInputPin(Step,'Souadnice 2',tpTPoint);
      AddOutputPin(Step,'Vsledn soadnice',tpTPoint);
  end;

  NasobekX:= 1;
  NasobekY:= 1;
  SpinEdit1Change(Self);
  LoadFromFile;

  ListView2Reload;
  if ZvolenaMetoda<ListView2.Items.Count then
    ListView2.ItemIndex:= ZvolenaMetoda else if ListView2.Items.Count>0 then ListView2.ItemIndex:= 0;
end;

{ TDetekce }

const
  Smery: array[0..3] of TPoint = ((X: 1; Y: 0),(X: 0; Y: -1),(X: -1; Y: 0),(X: 0; Y: 1));

procedure TDetekce.FinalPoint;
var
  Point: TPointType;
  OldPoint: TPoint;
  ControlMouse: TBooleanType;
  AvarageCount: TIntegerType;
  I: Integer;
begin
  Point:= TPointType(LocalInputPins[0]);
//  ControlMouse:= TBooleanType(LocalPropertyPins[0]);
//  AvarageCount:= TIntegerType(LocalPropertyPins[1]);
//  OldPoint:= TPointType(LocalPropertyPins[2]);

  with MainWindow do begin
    Label4.Caption:= 'Souadnice: X-'+IntToStr(Point.Value.X)+',Y-'+IntToStr(Point.Value.Y);
    // Ovldn plohy myy
    if Button6.Caption = '&Stop kalibrace' then begin
      if Point.Value.X<KalibraceMin.X then KalibraceMin.X:= Point.Value.X;
      if Point.Value.Y<KalibraceMin.Y then KalibraceMin.Y:= Point.Value.Y;
      if Point.Value.X>KalibraceMax.X then KalibraceMax.X:= Point.Value.X;
      if Point.Value.Y>KalibraceMax.Y then KalibraceMax.Y:= Point.Value.Y;
      //GetCursorPos(Stary);
      //WriteLn(SouborPoloh,IntToStr(Stred.X)+#9+IntToStr(Stary.X));
    end;
    if CheckBox5.Checked then begin
      //GetCursorPos(Stary);
      Point.Value.X:= Round((Point.Value.X-Posun.X)*NasobekX);
      Point.Value.Y:= Round((Point.Value.Y-Posun.Y)*NasobekY);
      //if((Abs(Stary.X-Novy.X)<50) and (Abs(Stary.Y-Novy.Y)<50)) then
      begin
        // Aktualizace prmru
        UkazatelPrumeru:= (UkazatelPrumeru+1) mod Length(Prumerovani);
        Prumerovani[UkazatelPrumeru].X:= Point.Value.X;
        Prumerovani[UkazatelPrumeru].Y:= Point.Value.Y;
        // Vpoet prmru
        OldPoint.X:= 0;
        OldPoint.Y:= 0;
        for I:= 0 to High(Prumerovani) do begin
          OldPoint.X:= OldPoint.X + Prumerovani[I].X;
          OldPoint.Y:= OldPoint.Y + Prumerovani[I].Y;
        end;
        OldPoint.X:= OldPoint.X div Length(Prumerovani);
        OldPoint.Y:= OldPoint.Y div Length(Prumerovani);
        SetCursorPos(OldPoint.X,Screen.Height-OldPoint.Y);
      end;
    end;
  end;
(*
  // Ovldn plohy myi
  if ControlMouse.Value then begin
    //GetCursorPos(Stary);
    Novy.X:= Round((Stred.X-Posun.X)*NasobekX);
    Novy.Y:= Round((Stred.Y-Posun.Y)*NasobekY);
    //if((Abs(Stary.X-Novy.X)<50) and (Abs(Stary.Y-Novy.Y)<50)) then
    begin
      // Aktualizace prmru
      OldPoint.Value.X:= (AvarageCount.Value-1)*OldPoint.Value.X+Point.Value.X) div AvarageCount.Value;
      OldPoint.Value.Y:= (AvarageCount.Value-1)*OldPoint.Value.Y+Point.Value.Y) div AvarageCount.Value;
      SetCursorPos(OldPoint.Value.X,OldPoint.Value.Y);
    end;
  end;
(*    if Button1.Caption = '&Stop kalibrace' then begin
      if Stred.X<KalibraceMin.X then KalibraceMin.X:= Stred.X;
      if Stred.Y<KalibraceMin.Y then KalibraceMin.Y:= Stred.Y;
      if Stred.X>KalibraceMax.X then KalibraceMax.X:= Stred.X;
      if Stred.Y>KalibraceMax.Y then KalibraceMax.Y:= Stred.Y;
      GetCursorPos(Stary);
      WriteLn(SouborPoloh,IntToStr(Stred.X)+#9+IntToStr(Stary.X));
    end;
*)
end;

procedure TDetekce.FinalPoint2;
var
  Point: TPointType;
  OldPoint, Kurzor: TPoint;
  ControlMouse: TBooleanType;
  AvarageCount: TIntegerType;
  I: Integer;
  Data: PChar;
begin
  Point:= TPointType(LocalInputPins[0]);
//  ControlMouse:= TBooleanType(LocalPropertyPins[0]);
//  AvarageCount:= TIntegerType(LocalPropertyPins[1]);
//  OldPoint:= TPointType(LocalPropertyPins[2]);

  with MainWindow do begin
    GetCursorPos(Kurzor);
    Bod:= Point.Value;
    Point.Value:= Aproxim.Transform(Point.Value);
    Label4.Caption:= 'Souadnice: ['+IntToStr(Bod.X)+','+IntToStr(Bod.Y)+'] => ['+IntToStr(Point.Value.X)+',Y1-'+IntToStr(Point.Value.Y)+']';
    if CheckBox5.Checked then begin
      //GetCursorPos(Stary);
      //if((Abs(Stary.X-Novy.X)<50) and (Abs(Stary.Y-Novy.Y)<50)) then
      begin
        // Aktualizace prmru
        UkazatelPrumeru:= (UkazatelPrumeru+1) mod Length(Prumerovani);
        Prumerovani[UkazatelPrumeru].X:= Point.Value.X;
        Prumerovani[UkazatelPrumeru].Y:= Point.Value.Y;
        // Vpoet prmru
        OldPoint.X:= 0;
        OldPoint.Y:= 0;
        for I:= 0 to High(Prumerovani) do begin
          OldPoint.X:= OldPoint.X + Prumerovani[I].X;
          OldPoint.Y:= OldPoint.Y + Prumerovani[I].Y;
        end;
        OldPoint.X:= OldPoint.X div Length(Prumerovani);
        OldPoint.Y:= OldPoint.Y div Length(Prumerovani);
        SetCursorPos(OldPoint.X,OldPoint.Y);
      end;
    end;
    if CheckBox6.Checked then with CommPort1 do begin
      ComNumber:= ComboBox1.ItemIndex+1;
      Open:= True;


      if Enabled then begin
        StatusBar1.Panels[2].Text:= ComboBox1.Items.Strings[ComboBox1.ItemIndex]+': OK';
        (* Odesln dat dc jednotce pes sriov port *)
        //Sleep(1000);
        Data:= PChar(IntToStr(Point.Value.X)+','+IntToStr(Point.Value.Y));
        PutBlock(Data,SizeOf(Data));
      end else StatusBar1.Panels[2].Text:= ComboBox1.Items.Strings[ComboBox1.ItemIndex]+': ERR';
      Open:= False;
    end else StatusBar1.Panels[2].Text:= ComboBox1.Items.Strings[ComboBox1.ItemIndex]+': OFF';
  end;
end;

function TDetekce.GetLocalInputPin(Index: Integer): Pointer;
begin
  with MainWindow, Methods[CurrentMethod].Steps[CurrentStep].Inputs do begin
    if Index>High(Pins) then ShowMessage('GetLocalInputPin: Tlo kroku '+AllStepsList[Methods[CurrentMethod].Steps[CurrentStep].StepType].Name+' poaduje nedefinovan parametr!')
    else begin
      //ShowMessage(IntToStr(Length(Pins))+'-'+IntToStr(Index));
      if Pins[Index]>High(PinValues) then SetLength(PinValues,Pins[Index]+1);
      with PinValues[Pins[Index]] do begin
        Typ:= AllStepsList[Methods[CurrentMethod].Steps[CurrentStep].StepType].InputsType[Index];
        if not Assigned(Data) then
          case Typ of
            tpInteger: Data:= TIntegerType.Create;
            tpReal: Data:= TRealType.Create;
            tpBoolean: Data:= TBooleanType.Create;
            tpTPoint: Data:= TPointType.Create;
            tpTScanImage: Data:= TScanImage.Create;
            tpTColor: Data:= TColorType.Create;
          end;
        Result:= Data;
      end;
    end;
  end;
end;

function TDetekce.GetLocalOutputPin(Index: Integer): Pointer;
begin
  with MainWindow, Methods[CurrentMethod].Steps[CurrentStep].Outputs do begin
    if Index>High(Pins) then ShowMessage('GetLocalOutputPin: Tlo kroku '+AllStepsList[Methods[CurrentMethod].Steps[CurrentStep].StepType].Name+' poaduje nedefinovan parametr!')
    else begin
      //ShowMessage(IntToStr(Length(Pins))+'-'+IntToStr(Index));
      if Pins[Index]>High(PinValues) then SetLength(PinValues,Pins[Index]+1);

      with PinValues[Pins[Index]] do begin
        Typ:= AllStepsList[Methods[CurrentMethod].Steps[CurrentStep].StepType].OutputsType[Index];
        if not Assigned(Data) then
          case Typ of
            tpInteger: Data:= TIntegerType.Create;
            tpReal: Data:= TRealType.Create;
            tpBoolean: Data:= TBooleanType.Create;
            tpTPoint: Data:= TPointType.Create;
            tpTScanImage: Data:= TScanImage.Create;
            tpTColor: Data:= TColorType.Create;
          end;
        Result:= Data;
      end;
    end;
  end;
end;

function TDetekce.GetLocalPropertyPin(Index: Integer): Pointer;
begin
  with MainWindow, Methods[CurrentMethod].Steps[CurrentStep] do
  Result:= Properties[Index].Data;
end;

function TDetekce.GetStepPinDataPointer(Index: Integer): Pointer;
begin

end;

procedure TDetekce.Histogram;
var
  Frame: TScanImage;
  Max: Integer;
  X, V, Y: Integer;
  Cetnosti: array[0..255] of Integer;
  Height: Integer;
begin
  Frame:= TScanImage(LocalInputPins[0]);
  for X:= 0 to 255 do Cetnosti[X]:= 0;
  with Frame do
    for Y:= 0 to Height-1 do
      for X:= 0 to Width-1 do begin
        Inc(Cetnosti[Pixels[Y,X]]);
//        UHistogram.Histogram.Canvas.Pixels[X,Y]:= Pixels[Y,X];
      end;
  Max:= 0;
  for X:= 0 to 253 do if Cetnosti[X]>Max then Max:= Cetnosti[X];
  Height:= UHistogram.Histogram.Height-30;
  UHistogram.Histogram.Label1.Caption:= IntToStr(Max);
  with UHistogram.Histogram do
  for X:= 0 to 255 do with Canvas do begin
    Lock;
    MoveTo(X,Height);
    Pen.Color:= clBlack;
    V:= Trunc(Cetnosti[X]/Max*Height*UHistogram.Histogram.TrackBar1.Position/100);
    if V>Height then V:= Height;
    LineTo(X,Height-V);
    Pen.Color:= clWhite;
    LineTo(X,0);
    UnLock;
  end;
end;

procedure TDetekce.HledaniBilehoCtverce;
var
  Frame, Frame2: TScanImage;
  X, Y, I: Integer;
  From: TPoint;
  Target: TPointType;
  Max: Byte;
begin
  Frame:= TScanImage(LocalInputPins[0]);
  Target:= TPointType(LocalOutputPins[0]);
  From.X:= Trunc(Frame.Width*0.75);
  From.Y:= Round(Frame.Height/5*2);
  Max:= 0;
  for Y:= From.Y to Frame.Height-1 do
    for X:= From.X to Trunc(Frame.Width/100*85) do
      if Max<Frame.Pixels[Frame.Height-Y,X] then begin
        Max:= Frame.Pixels[Frame.Height-Y,X];
        Target.Value.X:= X;
        Target.Value.Y:= Frame.Height-Y;
      end;
end;

procedure TDetekce.HledaniObjektuRozdilem;
var
  Frame, Frame2: TScanImage;
  X, Y, I: Integer;
  Value: Cardinal;
  InLevel, OutLevel, Half, Half2: TIntegerType;
begin
  Frame:= TScanImage(LocalInputPins[0]);
  Frame2:= TScanImage(LocalOutputPins[0]);
  Frame2.SetDimension(Frame.Width,Frame.Height);
  InLevel:= TIntegerType(LocalPropertyPins[0]);
  OutLevel:= TIntegerType(LocalPropertyPins[1]);
  Half:= TIntegerType(LocalPropertyPins[2]);
  Half2:= TIntegerType(LocalPropertyPins[3]);
  if Half2.Value<= Half.Value then Half2.Value:= Half.Value;
  with Frame do
  for Y:= Half2.Value to Height - Half2.Value - 1 do
    for X:= Half2.Value to Width - Half2.Value - 1 do begin
      Value:= Abs(Pixels[Y,X-1]-InLevel.Value);
      for I:= 0 to Half.Value-1 do
        Value:= Value + Abs(Pixels[Y+I,X]-InLevel.Value)+Abs(Pixels[Y-I,X]-InLevel.Value)+
        Abs(Pixels[Y,X+1]-InLevel.Value)+Abs(Pixels[Y,X-1]-InLevel.Value);
      for I:= Half.Value to Half2.Value-1 do
        Value:= Value + Abs(Pixels[Y+I,X]-OutLevel.Value)+Abs(Pixels[Y-I,X]-OutLevel.Value)+
        Abs(Pixels[Y,X+1]-OutLevel.Value)+Abs(Pixels[Y,X-1]-OutLevel.Value);
      Frame2.Pixels[Y,X]:= 255-Byte(Value div (Half2.Value*4+1));
    end;
end;

procedure TDetekce.HledaniObrysu;
var
  Konec, Ukoncit: Boolean;
  Start, Pos: TPoint;
  I, Smer: Integer;
  BodVeSmeru, BodVeStredu: Byte;
  Min, Max: TPoint;
  PuvodniSmer: Integer;
  PolohyX, PolohyY: array of Integer;
  N, P: Cardinal;
  Polomer: TPoint;
  Stred: TPointType;
  Stred2: TPointType;
  Frame, Frame2: TScanImage;
  IntTolerance: TIntegerType;
  Tolerance: Integer;
  MimoRozsah: Boolean;
begin
  Frame:= TScanImage(LocalInputPins[0]);
  Frame2:= TScanImage(LocalOutputPins[0]);
  Frame2.SetDimension(Frame.Width,Frame.Height);
  Move(Frame.Pixels,Frame2.Pixels,SizeOf(Frame.Pixels));
  Stred:= TPointType(LocalInputPins[1]);
  Stred2:= TPointType(LocalOutputPins[1]);
  IntTolerance:= TIntegerType(LocalPropertyPins[0]);
  Tolerance:= IntTolerance.Value;
 // try
  with Frame2 do begin
    PuvodniSmer:= 4;
    Ukoncit:= False;
    Pos.X:= Stred.Value.X;
    Pos.Y:= Stred.Value.Y;
    // Nalezen nejpravjho bodu
    Konec:= False;
    repeat
      //Body[Stred.Y,Pos.X]:= 255;
      Pos.X:= Pos.X + 1;
    until (Pos.X>(Width-3)) or ((Pixels[Stred.Value.Y,Pos.X]-Pixels[Stred.Value.Y,Stred.Value.X])>Tolerance);
    Pos.X:= Pos.X - 1;
    Start:= Pos;
    Max:= Pos;
    Min:= Pos;
    Smer:= 0;
    SetLength(PolohyX,Width);
    SetLength(PolohyY,Height);
    BodVeStredu:= Pixels[Stred.Value.Y,Stred.Value.X];
    repeat
      Pixels[Pos.Y,Pos.X]:= 255;     // Oznakuj aktualn bod
      PolohyX[Pos.X]:= PolohyX[Pos.X] + 1;
      PolohyY[Pos.Y]:= PolohyY[Pos.Y] + 1;
      if Pos.X>Max.X then Max.X:= Pos.X;
      if Pos.Y>Max.Y then Max.Y:= Pos.Y;
      if Pos.X<Min.X then Min.X:= Pos.X;
      if Pos.Y<Min.Y then Min.Y:= Pos.Y;
      // Najdi nov msto
      Konec:= False;
      I:= 0;
      repeat
        //R:= Body[Stred.Y,Stred.X];
        //S:= Body[Pos.Y+Smery[Smer].Y,Pos.X+Smery[Smer].X];
        if ((Pos.Y+Smery[Smer].Y)>=0) and ((Pos.Y+Smery[Smer].Y)<Height) and
        ((Pos.X+Smery[Smer].X)>=0) and ((Pos.X+Smery[Smer].X)<Width) then begin
          BodVeSmeru:= Pixels[Pos.Y+Smery[Smer].Y,Pos.X+Smery[Smer].X];
        end else MimoRozsah:= True;
        Konec:= ((Abs(BodVeSmeru-BodVeStredu)<=Tolerance) or (BodVeSmeru=255)) and not MimoRozsah;
        if not Konec then begin
          Smer:= (Smer+1) mod 4;
          I:= I + 1;
          if I>3 then Konec:= True;
        end;
      until Konec;
      Ukoncit:= ((Pos.Y=Start.Y)and(Pos.X=Start.X)and(PuvodniSmer=Smer)) or ((I>3)and(PuvodniSmer=4)) or MimoRozsah;
      if PuvodniSmer=4 then PuvodniSmer:= Smer;
      // Pejdi na novou souadnici
      Pos.X:= Pos.X + Smery[Smer].X;
      Pos.Y:= Pos.Y + Smery[Smer].Y;
      // Nato se zpt
      Smer:= Smer-1;
      if Smer<0 then Smer:= 3;
    until Ukoncit or (Pos.Y<1) or (Pos.Y>(Height-2)) or (Pos.X<1) or (Pos.X>(Width-2));

    Polomer.X:= (Max.X-Min.X) div 2;
    Polomer.Y:= (Max.Y-Min.Y) div 2;

    // Vpoet stedu prmrovnm
    N:= 0;
    P:= 0;
    for I:= 0 to Width-1 do begin
      N:= N + PolohyX[I]*I;
      P:= P + PolohyX[I];
    end;
    Stred2.Value.X:= N div P;
    N:= 0;
    P:= 0;
    for I:= 0 to Height-1 do begin
      N:= N + PolohyY[I]*I;
      P:= P + PolohyY[I];
    end;
    Stred2.Value.Y:= N div P;
  end;
  //except
  //end;
end;

procedure TDetekce.InitPins;
begin

end;

procedure TDetekce.InitPinValues;
begin
end;

procedure TDetekce.JasKontrast;
var
  X, Y, N: Integer;
  Max, Min: Byte;
  Auto: TBooleanType;
  Frame, Frame2: TScanImage;
  Kontrast: TRealType;
  Jas: TIntegerType;
begin
  Frame:= TScanImage(LocalInputPins[0]);
  Frame2:= TScanImage(LocalOutputPins[0]);
  Frame2.SetDimension(Frame.Width,Frame.Height);
  Kontrast:= TRealType(LocalPropertyPins[0]);
  Jas:= TIntegerType(LocalPropertyPins[1]);
  Auto:= TBooleanType(LocalPropertyPins[2]);
  if Auto.Value then begin
    Max:= 0;
    Min:= 255;
    for Y:= 0 to Height-1 do
    for X:= 0 to Width-1 do begin
      if Frame.Pixels[Y,X]>Max then Max:= Frame.Pixels[Y,X];
      if Frame.Pixels[Y,X]<Min then Min:= Frame.Pixels[Y,X];
    end;
    Kontrast.Value:= 255/(Max-Min+0.01);
    Jas.Value:= -Round(Min*Kontrast.Value);
  end;
  for Y:= 0 to Height-1 do
  for X:= 0 to Width-1 do begin
    N:= Trunc(Frame.Pixels[Y,X]*Kontrast.Value+Jas.Value);
    if N>255 then N:= 255;
    Frame2.Pixels[Y,X]:= N;
  end;
end;

procedure TDetekce.KontrastniPrechody;
var
  X, Y: Integer;
  N: Byte;
  IntSkok: TIntegerType;
  Skok: Integer;  // S jakm skokem se budou odetat pixely
  Frame, Frame2: TScanImage;
  Auto: TBooleanType;
begin
  Frame:= TScanImage(LocalInputPins[0]);
  Frame2:= TScanImage(LocalOutputPins[0]);
  Frame2.SetDimension(Frame.Width,Frame.Height);
  IntSkok:= TIntegerType(LocalPropertyPins[0]);
  Auto:= TBooleanType(LocalPropertyPins[1]);
  if Auto.Value then Skok:= Width div 100 else Skok:= IntSkok.Value;
  with Frame do
  for Y:= 0 to Height-1-Skok do
    for X:= 0 to Width-1-Skok do begin
      N:= Pixels[Y,X];
      Frame2.Pixels[Y,X]:= Byte((Abs(N-Pixels[Y,X+Skok])+Abs(N-Pixels[Y+Skok,X])) div 2);
    end;
end;

procedure TDetekce.NactiSnimek;
var
  DIBData: PDIBData;
  X, Y, XX, LastXX: Integer;
  BMax, Pixel: Integer;
  Radek: PByteArray;
  I: Integer;
  RX: Real;
  Frame: TScanImage;
  Fast: TBooleanType;
  Step, StepCount: Integer;
  BRest: Integer;
  PixelsAddr: Pointer;
  BufferAddr: Pointer;
  BufLen: Integer;
begin
  Frame:= TScanImage(LocalOutputPins[0]);
//  ShowMessage(IntToStr(SizeOf(Frame.Pixels)));
  Frame.Left:= 0;
  Frame.Top:= 0;
  Frame.SetDimension(Width,Height);
  Fast:= TBooleanType(LocalPropertyPins[0]);

  if Assigned(Buffer) then with Frame do begin
    if Fast.Value then begin
      // Dekoduj pmo z bufferu
      X:= Width;
      Y:= Height;
      BMax:= Trunc(Width*3/4);
      Step:= BMax div (Width-BMax);
      StepCount:= BMax div Step;
      BRest:= FrameWidth - X+1;
      BufLen:= 1024*Y+5000;
      BufferAddr:= Buffer;
      PixelsAddr:= Addr(Frame.Pixels);
      asm
        PUSH    ESI
        PUSH    EDI
        MOV     ESI, BufferAddr
        MOV     EDI, PixelsAddr
//      ADD     EDI, BufLen
        MOV     EDX, Y
      @loop1:
        MOV     EBX, StepCount
      @loop3:
        MOV     ECX, Step
      @loop2:
//      CLD
        LODSD
//      STD
        STOSB
        DEC     ECX
        JNZ     @loop2
        STOSB
        DEC     EBX
        JNZ     @loop3
        MOV     ECX, BRest
        ADD     EDI, ECX
        DEC     DX
        JNZ     @loop1
        POP     EDI
        POP     ESI
      end;
    end else begin
    // Dekoduj pmo z bufferu
    //Label4.Caption:= IntToStr(Length(Radek^));
    BMax:= Trunc(Width*3/4);
    DIBData:= PDIBData(Buffer);
    begin
      I:= 0;
      for Y:= Height-1 downto 0 do begin
        LastXX:= -1;
        for X:= 0 to BMax-1 do begin
          XX:= Trunc(X*4/3);
          Pixel:= DIBData^[I] and $ff; // (SnimekY-1-Y)*BMax+X
          I:= I + 1;
          Pixels[Height-Y,XX]:= Pixel;
          if XX<>(LastXX+1) then Pixels[Height-Y,XX-1]:= (Pixel+Pixels[Height-Y,XX-2]) div 2;
          LastXX:= XX;
        end;
      end;
    end;
    end;
  end;
  //LocalOutputPins[0]:= Frame;
end;

procedure TDetekce.NactiSnimek2;
var
  X, Y: Integer;
  BMax, BRest: Integer;

  Radek: PByteArray;
  Step, StepCount: Integer;
  Frame: TScanImage;
  PixelsAddr: Pointer;
  BufferAddr: Pointer;
  BufLen: Integer;
begin
  Frame:= TScanImage(LocalOutputPins[0]);
  Frame.Left:= 0;
  Frame.Top:= 0;
  Frame.SetDimension(Width,Height);

  if Assigned(Buffer) then with Frame do begin
    // Dekoduj pmo z bufferu
    X:= Width;
    Y:= Height;
    BMax:= Trunc(Width*3/4);
    Step:= BMax div (Width-BMax);
    StepCount:= BMax div Step;
    BRest:= FrameWidth - X+1;
    BufLen:= 1024*Y+5000;
    BufferAddr:= Buffer;
    PixelsAddr:= Addr(Frame.Pixels);
    asm
      PUSH    ESI
      PUSH    EDI
      MOV     ESI, BufferAddr
      MOV     EDI, PixelsAddr
//      ADD     EDI, BufLen
      MOV     EDX, Y
    @loop1:
      MOV     EBX, StepCount
    @loop3:
      MOV     ECX, Step
    @loop2:
//      CLD
      LODSD
//      STD
      STOSB
      DEC     ECX
      JNZ     @loop2
      STOSB
      DEC     EBX
      JNZ     @loop3
      MOV     ECX, BRest
      ADD     EDI, ECX
      DEC     DX
      JNZ     @loop1
      POP     EDI
      POP     ESI
    end;
  end;
end;

procedure TDetekce.PolohaBarvy;
var
  X, Y: Integer;
  Barva: Byte;
  Frame: TScanImage;
  Stred: TPointType;
  Rozdil: Byte;
begin
  Frame:= TScanImage(LocalInputPins[0]);
  Stred:= TPointType(LocalOutputPins[0]);
  Barva:= TIntegerType(LocalPropertyPins[0]).Value;
  Rozdil:= 255;
  for Y:= 0 to Height-1 do begin
    for X:= 0 to Width-1 do begin
      if Rozdil>Abs(Barva-Frame.Pixels[Y,X]) then begin
        Rozdil:= Abs(Barva-Frame.Pixels[Y,X]);
        Stred.Value.X:= X;
        Stred.Value.Y:= Y;
      end;
    end;
  end;
  LocalOutputPins[0]:= Pointer(Stred);
end;

procedure TDetekce.Prahovani;
var
  Frame, Frame2: TScanImage;
  LowLevel, HighLevel: TIntegerType;
  X, Y: Integer;
begin
  Frame:= TScanImage(LocalInputPins[0]);
  Frame2:= TScanImage(LocalOutputPins[0]);
  Frame2.SetDimension(Frame.Width,Frame.Height);
  LowLevel:= TIntegerType(LocalPropertyPins[0]);
  HighLevel:= TIntegerType(LocalPropertyPins[1]);
  for X:= 0 to (Width-1) do
    for Y:= 0 to (Height-1) do
      if (Frame.Pixels[Y,X]>LowLevel.Value) and (Frame.Pixels[Y,X]<HighLevel.Value) then
        Frame2.Pixels[Y,X]:= 255 else Frame2.Pixels[Y,X]:= 0;
end;

procedure TDetekce.ProhoditSnimky;
//var
//  TempFrame: TScanImage;
begin
//  TempFrame:= Frame;
//  Frame:= Frame2;
//  Frame2:= TempFrame;
end;

procedure TDetekce.PrumerCtvercu;
var
  X, Y, XX, YY, I: Integer;
  N, MaxN: Cardinal;
  Krok, Okraj: Integer;
  Frame, Frame2: TScanImage;
  Stred: TPointType;
  Auto: TBooleanType;
  IntKrok: TIntegerType;
begin
  Frame:= TScanImage(LocalInputPins[0]);
  Frame2:= TScanImage(LocalOutputPins[0]);
  Frame2.SetDimension(Frame.Width,Frame.Height);
  Stred:= TPointType(LocalOutputPins[1]);
  IntKrok:= TIntegerType(LocalPropertyPins[0]);
  Auto:= TBooleanType(LocalPropertyPins[1]);
  if Auto.Value then Krok:= Width div 20 else Krok:= IntKrok.Value;
  MaxN:= High(Cardinal);
  Okraj:= 0;
  with Frame do
  for X:= 0 to ((Width-1-Okraj) div Krok)-1 do
    for Y:= 0 to ((Height-1-Okraj) div Krok)-1 do begin
      N:= 0;
      for XX:= 0 to Krok-1 do
      for YY:= 0 to Krok-1 do N:= N + Pixels[Okraj+Y*Krok+YY,Okraj+X*Krok+XX];
      if N<MaxN then begin
        MaxN:= N;
        Stred.Value.X:= X*Krok + Krok div 2;
        Stred.Value.Y:= Y*Krok + Krok div 2;
      end;
      N:= N div Krok div Krok;
      for XX:= 0 to Krok-1 do
      for YY:= 0 to Krok-1 do Frame2.Pixels[Y*Krok+YY,X*Krok+XX]:= Byte(N);
    end;
end;

procedure TDetekce.RozdilSnimku;
var
  X, Y: Integer;
  Frame, Frame2, Frame3: TScanImage;
begin
  Frame:= TScanImage(LocalInputPins[0]);
  Frame2:= TScanImage(LocalInputPins[1]);
  Frame3:= TScanImage(LocalOutputPins[0]);
  Frame3.SetDimension(Frame.Width,Frame.Height);
  for Y:=0 to Frame.Height-1 do
    for X:=0 to Frame.Width-1 do
      Frame3.Pixels[Y,X]:= Abs(Frame2.Pixels[Y,X]-Frame.Pixels[Y,X]);
end;

procedure TDetekce.RozdilSouradnic;
var
  Stred1, Stred2, Stred3: TPointType;
begin
  Stred1:= TPointType(LocalInputPins[0]);
  Stred2:= TPointType(LocalInputPins[1]);
  Stred3:= TPointType(LocalOutputPins[0]);
  Stred3.Value.X:= 500+Stred2.Value.X - Stred1.Value.X;
  Stred3.Value.Y:= 500+Stred2.Value.Y - Stred1.Value.Y;
end;

procedure TDetekce.SetLocalInputPin(Index: Integer; const Value: Pointer);
begin

end;

procedure TDetekce.SetLocalOutputPin(Index: Integer; const Value: Pointer);
begin
  with MainWindow, Methods[CurrentMethod].Steps[CurrentStep].Outputs do begin
    if Index>High(Pins) then ShowMessage('SetLocalOutputPin: Tlo kroku '+AllStepsList[Methods[CurrentMethod].Steps[CurrentStep].StepType].Name+'poaduje nedefinovan vstupn parametr!')
    else begin
      if Pins[Index]>High(PinValues) then SetLength(PinValues,Pins[Index]+1);
//      ShowMessage(IntToStr(Pins[Index]));
      with PinValues[Pins[Index]] do Data:= Value;
    end;
  end;
end;

procedure TDetekce.TmaveOkoli;
var
  X, Y, N: Cardinal;
  I: Integer;
  MinN: Integer;
  Rozmach: Integer;
  Delitel: Integer;
  Frame, Frame2: TScanImage;
  Stred: TPointType;
begin
  Frame:= TScanImage(LocalInputPins[0]);
    // Odhad velikosti zornice oka
    Rozmach:= Width div 10;  // Rozmach jako 10% z sirky, prumer je pak 2*Rozmach
    Delitel:= Rozmach*4+2;   // Pedpoet potu prmrovanch bod
    MinN:= High(Integer);
    for Y:= Rozmach to Height-Rozmach-1 do
    for X:= Rozmach to Width-Rozmach-1 do begin
      N:= 0;
      for I:= -Rozmach to Rozmach do N:= N + Frame.Pixels[Y-I,X] + Frame.Pixels[Y,X-I];
      Frame2.Pixels[Y,X]:= Byte(N div (Rozmach*4+2));
      N:= N div Delitel;
      if N<MinN then begin
        MinN:= N;
        Stred.Value.X:= X;
        Stred.Value.Y:= Y;
      end;
    end;
end;

procedure TDetekce.TmaveOkoli2;
var
  X, Y, N: Cardinal;
  Snimek2: TScanImage;
  MinN: Cardinal;
  Soucet, Delitel, Max: Cardinal;
  SoucetY: array of Cardinal;
  IntRozmach: TIntegerType;
  Rozmach: Integer;
  Frame, Frame2: TScanImage;
  Stred: TPointType;
  Auto: TBooleanType;
begin
  Frame:= TScanImage(LocalInputPins[0]);
  Frame2:= TScanImage(LocalOutputPins[0]);
  Frame2.SetDimension(Frame.Width,Frame.Height);
  Stred:= TPointType(LocalOutputPins[1]);
  IntRozmach:= TIntegerType(LocalPropertyPins[0]);
  Auto:= TBooleanType(LocalPropertyPins[1]);
  with Frame do begin
    //if (Width> 800) or (Height>600) then ShowMessage('velikost');
    // Odhad velikosti zornice oka
    if Auto.Value then Rozmach:= Width div 10  // Rozmach jako 10% z sirky, prumer je pak 2*Rozmach
    else Rozmach:= IntRozmach.Value;

    Delitel:= Rozmach*4+2;   // Pedpoet potu prmrovanch bod
    MinN:= High(Cardinal);   // Zani na maximu
    // Pprava souctu pro osu Y
    SetLength(SoucetY,Width-2*Rozmach);
    for X:= Rozmach to Width-Rozmach-1 do begin
      Soucet:= 0;
      Max:= 2*Rozmach-1;
      for Y:= 0 to Max do Soucet:= Soucet + Pixels[Y,X];  // Predpocet souctu linie
      SoucetY[X-Rozmach]:= Soucet;
    end;
    for Y:= Rozmach to Height-Rozmach-1 do begin
      // Posunut sout SoucetY
      for X:= Rozmach to Width-Rozmach-1 do begin
        SoucetY[X-Rozmach]:= SoucetY[X-Rozmach] + Pixels[Y+Rozmach,X];
      end;
      Soucet := 0;
      for X:= 0 to 2*Rozmach-1 do Soucet:= Soucet + Pixels[Y,X];  // Predpocet souctu linie
      for X:= Rozmach to Width-Rozmach-1 do begin
        Soucet:= Soucet + Pixels[Y,X+Rozmach];
        //for I:= -Rozmach to Rozmach do N:= N + Body[Y-I,X] + Body[Y,X-I];
        N:= (Soucet+SoucetY[X-Rozmach]) div Delitel;
        Frame2.Pixels[Y,X]:= Byte(N);
//        if (N>255) and (Delitel>140) then MainWindow.Canvas.Pixels[X+300,Y]:= N;
        if N<MinN then begin
          MinN:= N;
          Stred.Value.X:= X;
          Stred.Value.Y:= Y;
        end;
        Soucet:= Soucet - Frame.Pixels[Y,X-Rozmach];
      end;
      // Posunut sout SoucetY
      for X:= Rozmach to Width-Rozmach-1 do begin
        SoucetY[X-Rozmach]:= SoucetY[X-Rozmach] - Pixels[Y-Rozmach,X];
      end;
    end;
  end;
end;

procedure TDetekce.TmaveOkoliASM;
var
  X, Y, N: Cardinal;
  Snimek2: TScanImage;
  MinN: Cardinal;
  Soucet, Delitel, Max: Cardinal;
  SoucetY: array[0..1023] of Cardinal;
  IntRozmach: TIntegerType;
  Rozmach: Integer;
  Frame, Frame2: TScanImage;
  Stred: TPointType;
  Auto: TBooleanType;
  FrameAddr: Pointer;
  Frame2Addr: Pointer;
  SoucetAddr: Pointer;
  FWidth: Integer;
  Text: string;
begin
  Frame:= TScanImage(LocalInputPins[0]);
  Frame2:= TScanImage(LocalOutputPins[0]);
  Frame2.SetDimension(Frame.Width,Frame.Height);
  Stred:= TPointType(LocalOutputPins[1]);
  IntRozmach:= TIntegerType(LocalPropertyPins[0]);
  Auto:= TBooleanType(LocalPropertyPins[1]);
  with Frame do begin
    //if (Width> 800) or (Height>600) then ShowMessage('velikost');
    // Odhad velikosti zornice oka
    if Auto.Value then Rozmach:= Width div 10  // Rozmach jako 10% z sirky, prumer je pak 2*Rozmach
    else Rozmach:= IntRozmach.Value;

    Delitel:= Rozmach*4+2;   // Pedpoet potu prmrovanch bod
    MinN:= High(Cardinal);   // Zani na maximu
    // Pprava souctu pro osu Y
    FrameAddr:= Addr(Frame.Pixels);
    Frame2Addr:= Addr(Frame2.Pixels);
    SoucetAddr:= Addr(SoucetY);
    FWidth:= Frame.Width;
(*    for X:= Rozmach to Width-Rozmach-1 do begin
      Soucet:= 0;
      Max:= 2*Rozmach-1;
      for Y:= 0 to Max do Soucet:= Soucet + Pixels[Y,X];  // Predpocet souctu linie
      SoucetY[X-Rozmach]:= 0; // Soucet;
    end;
*)
    asm
      PUSH  ESI
      PUSH  EDI
      // Piprava souctu pro osu Y
      MOV   EDI, SoucetAddr
      MOV   ECX, 1024/4
      XOR   EAX, EAX
      STOSD
      MOV   ESI, FrameAddr
      ADD   ESI, Rozmach
      MOV   EDI, SoucetAddr
      MOV   ECX, FWidth
      SUB   ECX, Rozmach
      SUB   ECX, Rozmach
    @loop2:
      MOV   EBX, Rozmach
      SHL   EBX, 1
      PUSH  ESI
      MOV   EDX, 0
    @loop:
      MOV   EAX, [ESI]
      AND   EAX, 255
      ADD   EDX, EAX
      ADD   ESI, FrameWidth
      DEC   EBX
      JNZ   @loop
      MOV   [EDI], EDX
      ADD   EDI,4
      POP   ESI
      INC   ESI
      DEC   ECX
      JNZ   @loop2
      POP   EDI
      POP   ESI
    end;
//    Text:= '';
//    for X:= Rozmach to Width-Rozmach-1 do Text:= Text + IntToStr(SoucetY[X])+',';

    for Y:= Rozmach to Height-Rozmach-1 do begin
      // Posunut sout SoucetY
      for X:= Rozmach to Width-Rozmach-1 do begin
        SoucetY[X-Rozmach]:= SoucetY[X-Rozmach] + Pixels[Y+Rozmach,X];
      end;
      Soucet := 0;
      for X:= 0 to 2*Rozmach-1 do Soucet:= Soucet + Pixels[Y,X];  // Predpocet souctu linie
      for X:= Rozmach to Width-Rozmach-1 do begin
        Soucet:= Soucet + Pixels[Y,X+Rozmach];
        //for I:= -Rozmach to Rozmach do N:= N + Body[Y-I,X] + Body[Y,X-I];
        N:= (Soucet+SoucetY[X-Rozmach]) div Delitel;
        Frame2.Pixels[Y,X]:= Byte(N);
//        if (N>255) and (Delitel>140) then MainWindow.Canvas.Pixels[X+300,Y]:= N;
        if N<MinN then begin
          MinN:= N;
          Stred.Value.X:= X;
          Stred.Value.Y:= Y;
        end;
        Soucet:= Soucet - Frame.Pixels[Y,X-Rozmach];
      end;
      // Posunut sout SoucetY
      for X:= Rozmach to Width-Rozmach-1 do begin
        SoucetY[X-Rozmach]:= SoucetY[X-Rozmach] - Pixels[Y-Rozmach,X];
      end;
    end;
    
  end;
end;

procedure TDetekce.Vyhlazeni;
var
  X, Y: Integer;
  Frame, Frame2: TScanImage;
begin
  Frame:= TScanImage(LocalInputPins[0]);
  Frame2:= TScanImage(LocalOutputPins[0]);
  Frame2.SetDimension(Frame.Width,Frame.Height);
    for Y:= 1 to Height-1 do
      for X:= 1 to Width-1 do
        Frame2.Pixels[Y,X]:= (Frame.Pixels[Y,X-1]+Frame.Pixels[Y-1,X]+Frame.Pixels[Y,X]) div 3;
 // LocalOutputPins[0]:= Frame2;
end;

procedure TDetekce.ZamereniKrizem;
var
  Frame, Frame2: TScanImage;
  X, Y: Integer;
  Pos: TPoint;
  PosMin, PosMax: Integer;
  Konec: Boolean;
  NovyStred: TPoint;
  Stred, Stred2: TPointType;
  IntTolerance: TIntegerType;
  Tolerance: Integer;
  Polomer: TPoint;
begin
  Frame:= TScanImage(LocalInputPins[0]);
  Frame2:= TScanImage(LocalOutputPins[0]);
  Frame2.SetDimension(Frame.Width,Frame.Height);
  Move(Frame.Pixels,Frame2.Pixels,SizeOf(Frame.Pixels));
  Stred:= TPointType(LocalInputPins[1]);
  Stred2:= TPointType(LocalOutputPins[1]);
  IntTolerance:= TIntegerType(LocalPropertyPins[0]);
  Tolerance:= IntTolerance.Value;
  with Frame2 do begin
    // Najdi sted na ose Y
    // PRchod smrem dol
    Konec:= False;
    Pos.Y:= Stred.Value.Y;
    while not Konec do begin
      Konec:= (Pos.Y>=Height) or ((Pixels[Pos.Y,Stred.Value.X]-Pixels[Stred.Value.Y,Stred.Value.X])>Tolerance);
      Pos.Y:= Pos.Y + 1;
    end;
    PosMax:= Pos.Y;
    // Pruchod nahoru
    Konec:= False;
    Pos.Y:= Stred.Value.Y;
    while not Konec do begin
      Konec:= (Pos.Y=0) or ((Pixels[Pos.Y,Stred.Value.X]-Pixels[Stred.Value.Y,Stred.Value.X])>Tolerance);
      Pos.Y:= Pos.Y - 1;
    end;
    PosMin:= Pos.Y;
    // Vpoet novho stedu
    Polomer.Y:= (PosMax - PosMin) div 2;
    NovyStred.Y:= PosMin + Polomer.Y;

    // Najdi sted na ose X
    // Prchod smrem dol
    Konec:= False;
    Pos.X:= Stred.Value.X;
    while not Konec do begin
      Konec:= (Pos.X>=Width) or ((Pixels[Stred.Value.Y,Pos.X]-Pixels[Stred.Value.Y,Stred.Value.X])>Tolerance);
      Pos.X:= Pos.X + 1;
    end;
    PosMax:= Pos.X;
    // Pruchod nahoru
    Konec:= False;
    Pos.X:= Stred.Value.X;
    while not Konec do begin
      Konec:= (Pos.X=0) or ((Pixels[Stred.Value.Y,Pos.X]-Pixels[Stred.Value.Y,Stred.Value.X])>Tolerance);
      Pos.X:= Pos.X - 1;
    end;
    PosMin:= Pos.X;
    // Vpoet novho stedu
    Polomer.X:= (PosMax - PosMin) div 2;
    NovyStred.X:= PosMin + Polomer.X;
    Stred2.Value:= NovyStred;
  end;
end;

procedure TDetekce.ZobrazKriz;
var
  Frame, Frame2: TScanImage;
  Stred: TPointType;
  CrossColor: TColorType;
begin
  Stred:= TPointType(LocalInputPins[0]);
  CrossColor:= TColorType(LocalPropertyPins[0]);
  with Preview.Canvas do begin
    Lock;
    Pen.Color:= CrossColor.Color;
    MoveTo(Round(Stred.Value.X/Width*Preview.Width)-10,0);
    LineTo(Round(Stred.Value.X/Width*Preview.Width)-10,Preview.Height-20);
    MoveTo(0,Preview.Height-Round(Stred.Value.Y/Height*(Preview.Height-20))-20);
    LineTo(Preview.Width,Preview.Height-Round(Stred.Value.Y/Height*(Preview.Height-20))-20);
    UnLock;
  end;
end;

procedure TDetekce.ZobrazSnimek;
var
  DIBData: PDIBData;
  X, Y: Integer;
  Frame: TScanImage;
  BufferAddr, PixelsAddr: Pointer;
  Pocet, Rest, Sirka, Vyska, Res: Integer;
  FastDraw: TBooleanType;
  KX, KY: Real;
const
  Krok = 1;
begin
  Frame:= TScanImage(LocalInputPins[0]);
  FastDraw:= TBooleanType(LocalPropertyPins[0]);
  if MainWindow.CheckBox1.Checked then
  with MainWindow, Preview, Detekce, Frame do
  try
    Canvas.Lock;
    if FastDraw.Value then begin
    if (Bitmapa.Width<>Width) or (Bitmapa.Height<>Height) then begin
      Bitmapa.Width:= Width;
      Bitmapa.Height:= Height;
      InternalGetDIBSizes(Bitmapa.Handle, BitmapInfoSize, BitmapBitsSize, 0);
      FreeMem(Bitmap);
      Bitmap := AllocMem(DWORD(BitmapInfoSize) + BitmapBitsSize);
      Bits := Pointer(DWORD(Bitmap) + BitmapInfoSize);
      InternalGetDIB(Bitmapa.Handle, 0, Bitmap^, Bits^, 0);
      DIBData:= PDIBData(Bits);
      //ShowMessage(IntToStr(Width)+','+IntToStr(Height));
    end;
    BufferAddr:= Bits;
    PixelsAddr:= Addr(Frame.Pixels);
    Pocet:= Frame.Width*Frame.Height;
    Rest:= 1024-Frame.Width;
    Sirka:= Frame.Width;
    Vyska:= Frame.Height;
    asm
      PUSH    ESI
      PUSH    EDI
      MOV     EDI, BufferAddr
      MOV     ESI, PixelsAddr
      MOV     EDX, Vyska
    @loop2:
      MOV     ECX, Sirka
    @loop:
      LODSB
      STOSB
      STOSB
      STOSB
      XOR     AL,AL
      STOSB
      DEC     ECX
      JNZ     @loop
      ADD     ESI,Rest
      DEC     EDX
      JNZ     @loop2
      POP     EDI
      POP     ESI
    end;

    Res:= StretchDIBits(Canvas.Handle,0,0,Preview.Width-10,Preview.Height-20,0,0,Width,Height,Bits,PBitmapInfo(Bitmap)^,DIB_RGB_COLORS,SrcCopy);
    MainWindow.Label6.Caption:= IntToStr(Res);
    if Res = 0 then begin
      MainWindow.Label6.Caption:= SysErrorMessage(GetLastError);
    end;
    end else begin
      (*
      if Canvas.Pixels[10*Krok+1,10*Krok+1]<>clGray then begin
        Canvas.Brush.Color:= clGray;
        Canvas.FillRect(Rect(0,0,Width,Height));
      end;
      *)
      KX:= Width/(Preview.Width-10);
      KY:= Height/(Preview.Height-20);
      for Y:= 0 to (Preview.Height-1) div Krok do begin
        for X:= 0 to (Preview.Width-1) div Krok do begin
          Canvas.Pixels[X*Krok,Preview.Height-20 - Y*Krok]:= Frame.Pixels[Round(KY*Y*Krok),Round(KX*X*Krok)]*$010101;
        end;
      end;
    end;
  finally
    Canvas.UnLock;
  end;
end;

function GetActualTime: Integer;
var
  Frequency, Counter: Int64;
begin
  QueryPerformanceCounter(Counter);
  QueryPerformanceFrequency(Frequency);
  Result:= Counter*1000 div Frequency;
end;

procedure TMainWindow.SampleGrabber1Buffer(sender: TObject;
  SampleTime: Double; pBuffer: Pointer; BufferLen: Integer);
var
  I: Integer;
  StartTime, TotalTime: Integer;
begin
  if (not Zamek) and VideoEnabled then with Detekce do begin
    Zamek:= True;

    Detekce.Buffer:= pBuffer;
    Detekce.BufferLength:= BufferLen;

    // Prove kroky
    if Assigned(ListView2.Selected) then
    with Methods[ListView2.Selected.Index] do begin
      CurrentMethod:= ListView2.Selected.Index;
      TotalTime:= 0;
      for I:= 0 to High(Steps) do begin
        StartTime:= GetActualTime;
        CurrentStep:= I;
        AllStepsList[Steps[I].StepType].Call;
        StartTime:= GetActualTime - StartTime;
        StartTime:= (StrToInt(ListView1.Items.Item[I].SubItems[1])*5+StartTime) div 6;
        TotalTime:= TotalTime + StartTime;
        ListView1.Items.Item[I].SubItems[1]:= IntToStr(StartTime);
      end;

      // Vypoti souet as
      with ListView1.Items do Item[Count-1].SubItems[1]:= IntToStr(TotalTime);
    end;

    Zamek:= False;
  end;
end;

function TMainWindow.PridatMetodu(Nazev: string): TMetoda;
begin
  SetLength(Methods,Length(Methods)+1);
  Methods[High(Methods)]:= TMetoda.Create;
  Methods[High(Methods)].Nazev:= Nazev;
  Result:= Methods[High(Methods)];
end;

procedure TMetoda.PridatKrok(Step: Integer);
begin
  SetLength(Steps,Length(Steps)+1);
  Steps[High(Steps)]:= TStep.Create;
  Steps[High(Steps)].StepType:= Step;
end;

procedure TMainWindow.ListView2SelectItem(Sender: TObject; Item: TListItem;
  Selected: Boolean);
var
  I: Integer;
  Node: TListItem;
begin
  if Assigned(ListView2.Selected) then
  // Zobrazit seznam krok
  with ListView1.Items, Methods[ListView2.ItemIndex] do begin
    VideoEnabled:= False;
    while Zamek do Application.ProcessMessages;
    BeginUpdate;
    Clear;
    for I:= 0 to High(Steps) do begin
      Node:= Add;
      Node.Caption:= IntToStr(I);
      with Node.SubItems do begin
        Add(AllStepsList[Steps[I].StepType].Name);
        Add('0');
      end;
    end;
    Node:= Add;
    Node.Caption:= '';
    with Node.SubItems do begin
      Add('Celkem');
      Add('0');
    end;
    EndUpdate;
    ZvolenaMetoda:= ListView2.ItemIndex;
    with Detekce do for I:=0 to High(PinValues) do TObject(PinValues[I].Data).Free;
    SetLength(Detekce.PinValues,0);
    VideoEnabled:= True;
    if Assigned(PropertiesForm) then PropertiesForm.ResetFrames;
  end;
end;

procedure TMainWindow.FormClose(Sender: TObject; var Action: TCloseAction);
var
  I: Integer;
begin
  ZdrojTyp:= Options.RadioButton1.Checked;
  ZobrazitNahled:= CheckBox1.Checked;
  ProvadetFiltry:= CheckBox2.Checked;
  ShowHistogram:= CheckBox7.Checked;
  Soubor:= Options.Memo1.Lines.Strings[0];
  Registry.Free;
  StopVideo;
  SysDev.Free;
  ListView2.Items.Clear;
  ListView1.Items.Clear;
  SaveToFile;
  for I:= 0 to High(Methods) do Methods[I].Free;
  Detekce.Aproxim.Free;
  Detekce.Free;
end;

procedure TMainWindow.CheckBox1Click(Sender: TObject);
begin
  if Assigned(Preview) then
    if CheckBox1.Checked then Preview.Show else Preview.Hide;
  ZobrazitNahled:= CheckBox1.Checked;
end;

procedure TMainWindow.CheckBox2Click(Sender: TObject);
begin
  VideoEnabled:= CheckBox2.Checked;
  ProvadetFiltry:= CheckBox2.Checked;
end;

procedure TMainWindow.StopVideo;
begin
  VideoEnabled:= False;
  while Zamek do Application.ProcessMessages;
  FilterGraph1.ClearGraph;
  FilterGraph1.Active := False;
end;

procedure TMainWindow.CheckBox3Click(Sender: TObject);
begin
  PreviewOnTop:= CheckBox3.Checked;
  if Assigned(Preview) then
    if PreviewOnTop then Preview.FormStyle:= fsStayOnTop else Preview.FormStyle:= fsNormal;
end;

procedure TMainWindow.FormShow(Sender: TObject);
begin
  if PreviewOnTop then Preview.FormStyle:= fsStayOnTop else Preview.FormStyle:= fsNormal;
end;

procedure TMainWindow.Zobraz;
begin
  if Assigned(ListView2.Selected) then
  SampleGrabber1Buffer(Self,0,nil,0);
end;

{ TPinArray }

procedure TPinArray.AddPin(Index: Integer);
begin
  SetLength(Pins,Length(Pins)+1);
  Pins[High(Pins)]:= Index;
end;

procedure TMainWindow.Button5Click(Sender: TObject);
begin
  Edit.ShowModal;
end;

{ TStep }

constructor TStep.Create;
begin
  inherited;
  Outputs:= TPinArray.Create;
  Inputs:= TPinArray.Create;
//  Properties:= TPinArray.Create;
end;

function TStep.StepName: string;
begin
  Result:= MainWindow.AllStepsList[Integer(StepType)].Name;
end;

function TMainWindow.AddStep(Name: string; Call: TMethod;
  ProcessStepType: TProcessStepType): TStepDef;
begin
  SetLength(AllStepsList,Length(AllStepsList)+1);
  AllStepsList[High(AllStepsList)]:= TStepDef.Create;
  AllStepsList[High(AllStepsList)].Name:= Name;
  AllStepsList[High(AllStepsList)].Call:= Call;
  AllStepsList[High(AllStepsList)].ProcessStepType:= ProcessStepType;
  Result:= AllStepsList[High(AllStepsList)];
end;

procedure TMainWindow.SaveToFile;
var
  ConfigFile: TextFile;
  StepsStr, StrData: string;
  I, J, K: Integer;
begin
  AssignFile(ConfigFile, ConfigFileName);
  Rewrite(ConfigFile);
  for I:= 0 to High(Methods) do with Methods[I] do begin
    StepsStr:= '';
    for J:= 0 to High(Steps) do with Steps[J] do begin
      for K:= 0 to High(Inputs.Pins) do StepsStr:= StepsStr + IntToStr(Inputs.Pins[K]) + ',';
      StepsStr:= StepsStr + '>' + IntToStr(StepType)+'(';
      for K:= 0 to High(Properties) do with Properties[K] do begin
        StrData:= '';
        case Typ of
          tpInteger: StrData:= IntToStr(TIntegerType(Data).Value);
          tpReal: begin
            StrData:= FloatToStr(TRealType(Data).Value);
            if Pos(',',StrData)>0 then StrData[Pos(',',StrData)]:= '.';
          end;
          tpBoolean: StrData:= BoolToStr(TBooleanType(Data).Value);
          tpTColor: StrData:= ColorToString(TColorType(Data).Color)
        end;
        StepsStr:= StepsStr + StrData + ',';
      end;
      StepsStr:= StepsStr + ')>';
      for K:= 0 to High(Outputs.Pins) do StepsStr:= StepsStr +IntToStr(Outputs.Pins[K]) + ',';
      StepsStr:= StepsStr + ';';
    end;
    WriteLn(ConfigFile,Methods[I].Nazev+':'+StepsStr);
  end;
  CloseFile(ConfigFile);
end;

function Parse(var Text:string; Znak: Char): string;
begin
  Result:= Copy(Text,1,Pos(Znak,Text)-1);
  Delete(Text,1,Length(Result)+1);
end;

procedure TMainWindow.LoadFromFile;
var
  ConfigFile: TextFile;
  I: Integer;
  Row, Data, StrData: string;
begin
  try
    AssignFile(ConfigFile,ConfigFileName);
    if FileExists(ConfigFileName) then Reset(ConfigFile) else Rewrite(ConfigFile);
    while not Eof(ConfigFile) do begin
      ReadLn(ConfigFile,Row);
      SetLength(Methods,Length(Methods)+1);
      Methods[High(Methods)]:= TMetoda.Create;
      Methods[High(Methods)].Nazev:= Parse(Row,':');
      while Length(Row)>0 do with Methods[High(Methods)] do begin
        SetLength(Steps,Length(Steps)+1);
        Steps[High(Steps)]:= TStep.Create;
        with Steps[High(Steps)] do begin
          Data:= Parse(Row,'>');
          while Length(Data)>0 do begin
            Inputs.AddPin(StrToInt(Parse(Data,',')));
          end;
          Data:= Parse(Row,'(');
          StepType:= StrToInt(Data);
          Data:= Parse(Row,')');
          SetLength(Properties,Length(AllStepsList[StepType].PropertiesType));
          for I:= 0 to High(AllStepsList[StepType].PropertiesType) do begin
            StrData:= Parse(Data,',');
            with Properties[I] do begin
              Typ:= AllStepsList[StepType].PropertiesType[I].PinType;
              case Typ of
              tpInteger: begin
                Data:= TIntegerType.Create;
                TIntegerType(Data).Value:= StrToInt(StrData);
              end;
              tpReal: begin
                Pointer(Data):= TRealType.Create;
                if Pos('.',StrData)>0 then StrData[Pos('.',StrData)]:= ',';
                TRealType(Data).Value:= StrToFloat(StrData);
              end;
              tpBoolean: begin
                Pointer(Data):= TBooleanType.Create;
                TBooleanType(Data).Value:= StrToBool(StrData);
              end;
              tpTColor: begin
                Pointer(Data):= TColorType.Create;
                TColorType(Data).Color:= StringToColor(StrData);
              end;
              end;
            end;
          end;
          Data:= Parse(Row,'>');
          Data:= Parse(Row,';');
          while Length(Data)>0 do begin
            Outputs.AddPin(StrToInt(Parse(Data,',')));
          end;
        end;
      end;
    end;
  finally
    CloseFile(ConfigFile);
  end;
end;

procedure TMainWindow.AddProperty(var Step: TStepDef; Text: string; PinType: TPinType);
begin
  with Step do begin
    SetLength(PropertiesType,Length(PropertiesType)+1);
    PropertiesType[High(PropertiesType)]:= TPropertyType.Create;
    PropertiesType[High(PropertiesType)].PinType:= PinType;
    PropertiesType[High(PropertiesType)].Text:= Text;
  end;
end;

function TMainWindow.PinTypeToStr(PinType: TPinType): string;
begin
end;

procedure TMainWindow.CheckBox4Click(Sender: TObject);
begin
  ShowPropertiesWindow:= CheckBox4.Checked;
  if Assigned(PropertiesForm) then
    if CheckBox4.Checked then PropertiesForm.Show else PropertiesForm.Hide;
end;

procedure TMainWindow.ListView1SelectItem(Sender: TObject; Item: TListItem;
  Selected: Boolean);
begin
  PropertiesForm.ResetFrames;
end;

procedure TMainWindow.ListView2Reload;
var
  I: Integer;
  Node: TListItem;
  PreviousSelected: Integer;
begin
  // Zobrazit seznam metod
  with ListView2.Items do begin
    PreviousSelected:= ListView2.ItemIndex;
    BeginUpdate;
    Clear;
    for I:= 0 to High(Methods) do begin
      Node:= Add;
      Node.Caption:= Methods[I].Nazev;
    end;
    EndUpdate;
    if (PreviousSelected>=0) and (PreviousSelected<ListView2.Items.Count) then
      ListView2.ItemIndex:= PreviousSelected else
        if ListView2.Items.Count>0 then  ListView2.ItemIndex:= 0;
  end;
end;

{ TScanImage }

procedure TScanImage.SetDimension(X, Y: Integer);
begin
  if (Width<>X) or (Height<>Y) then begin
    Width:= X;
    Height:= Y;
//    SetLength(Pixels,0,0);
//    SetLength(Pixels,Height,Width);
  end;
end;

procedure TMainWindow.AddInputPin(var Step: TStepDef; Text: string;
  PinType: TPinType);
begin
  with Step do begin
    SetLength(InputsType,Length(InputsType)+1);
    InputsType[High(InputsType)]:= PinType;
  end;
end;

procedure TMainWindow.AddOutputPin(var Step: TStepDef; Text: string;
  PinType: TPinType);
begin
  with Step do begin
    SetLength(OutputsType,Length(OutputsType)+1);
    OutputsType[High(OutputsType)]:= PinType;
  end;
end;

procedure TMainWindow.Timer1Timer(Sender: TObject);
var
  HeapStatus: THeapStatus;
begin
  HeapStatus:= GetHeapStatus;
  StatusBar1.Panels[0].Text:= 'Vyhrazen pam: '+IntToStr(HeapStatus.TotalCommitted);
  StatusBar1.Panels[1].Text:= 'Pouit pam: '+IntToStr(HeapStatus.TotalAllocated);
end;

procedure TMainWindow.Button6Click(Sender: TObject);
begin
  with Button6 do
  if Caption = '&Start kalibrace' then begin
    //AssignFile(SouborPoloh,'Polohy.txt');
    //Rewrite(SouborPoloh);
    Caption:= '&Stop kalibrace';
    KalibraceMin.X:= Detekce.Width;
    KalibraceMin.Y:= Detekce.Height;
    KalibraceMax.X:= 0;
    KalibraceMax.Y:= 0;
  end else begin
    //CloseFile(SouborPoloh);
    Caption:= '&Start kalibrace';
    Posun.X:= KalibraceMax.X;
    Posun.Y:= KalibraceMin.Y;
    NasobekX:= Screen.Width/(KalibraceMin.X-KalibraceMax.X);
    NasobekY:= Screen.Height/(KalibraceMax.Y-KalibraceMin.Y);
  end;
end;

procedure TMainWindow.SpinEdit1Change(Sender: TObject);
begin
  SetLength(Prumerovani,SpinEdit1.Value);
  UkazatelPrumeru:= 0;
end;

procedure TMainWindow.CheckBox5Click(Sender: TObject);
var
  Pozice: TPoint;
  I: Integer;                     
begin
  GetCursorPos(Pozice);
  for I:= 0 to High(Prumerovani) do begin
    Prumerovani[I].X:= Pozice.X;
    Prumerovani[I].Y:= Pozice.Y;
  end;
end;

procedure TMainWindow.Button8Click(Sender: TObject);
begin
  Detekce.Aproxim.Reset;
  SpinEdit2.Value:= Detekce.Aproxim.Pocet;
  Transform.FormPaint(Self);
end;

procedure TMainWindow.Button7Click(Sender: TObject);
var
  Kurzor: TPoint;
begin
  GetCursorPos(Kurzor);
  Detekce.Aproxim.Pridej(Detekce.Bod.X,Detekce.Bod.Y,Kurzor.X,Kurzor.Y);
  Detekce.Aproxim.Aproximuj;
  SpinEdit2.Value:= Detekce.Aproxim.Pocet;
  Transform.FormPaint(Self);
end;

procedure TMainWindow.Button9Click(Sender: TObject);
begin
  Transform.Show;
end;

procedure TMainWindow.CheckBox7Click(Sender: TObject);
begin
  if Assigned(UHistogram.Histogram) then
    if CheckBox7.Checked then UHistogram.Histogram.Show else UHistogram.Histogram.Hide;
  ZobrazitNahled:= CheckBox1.Checked;
end;

end.
