unit UPropertiesForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Spin, ComCtrls, ExtCtrls;

type
  TPropertiesForm = class(TForm)
    ColorDialog1: TColorDialog;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
  public
    Items: array of TComponent;
    { Public declarations }
    UpdatingList: Boolean;
    InitializingList: Boolean;
    procedure ResetFrames;
    procedure ReReadData;
    procedure ShapeMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure TrackBarChange(Sender: TObject);
    procedure SpinEditChange(Sender: TObject);
    procedure CheckBoxClick(Sender: TObject);
  end;

var
  PropertiesForm: TPropertiesForm;

implementation

uses UMainWindow, UIntegerFrame, UColorFrame, Math, UBooleanFrame,
  UPreview, URealFrame;

{$R *.dfm}

procedure TPropertiesForm.ResetFrames;
var
  I, II: Integer;
  ItemTop: Integer;
begin
  InitializingList:= True;
  while UpdatingList do Application.ProcessMessages;    // Wait to properties list updating finished

  ItemTop:= 0;
  II:= 0;
  with MainWindow do
    if Assigned(ListView1.Selected) and Assigned(ListView2.Selected) and
    (ListView1.Selected.SubItems.Strings[0]<>'Celkem') then
    with Methods[ListView2.Selected.Index].Steps[ListView1.Selected.Index] do begin
      for I:= 0 to High(Items) do Items[I].Free;    // Free previous components
      SetLength(Items,Length(Properties)*2);
      for I:= 0 to High(Properties) do begin
        Items[II]:= TLabel.Create(Self);
        with Items[II] as TLabel do begin
          Caption:= AllStepsList[StepType].PropertiesType[I].Text+':';
          Parent:= Self;
          Top:= ItemTop;
          Left:= 0;
          //Width:= PropertiesForm.Width-10;
          //Height:= 60;
          ItemTop:= ItemTop + 16;
        end;
        II:= II + 1;
        case Properties[I].Typ of
          tpInteger: begin
            SetLength(Items,Length(Items)+2);
            Items[II]:= TTrackBar.Create(Self);
            with Items[II] as TTrackBar do begin
              Parent:= Self;
              Top:= ItemTop;
              Left:= 0;
              Width:= PropertiesForm.Width-65;
              Height:= 30;
              Position:= TIntegerType(Properties[I].Data).Value;
              Min:= 1;
              Max:= 255;
              Frequency:= 10;
              OnChange:= TrackBarChange;
            end;
            II:= II + 1;
            Items[II]:= TSpinEdit.Create(Self);
            with Items[II] as TSpinEdit do begin
              Parent:= Self;
              Top:= ItemTop;
              ItemTop:= ItemTop + 35;
              Left:= PropertiesForm.Width-60;
              Width:= 50;
              Height:= 22;
              Value:= TIntegerType(Properties[I].Data).Value;
              MinValue:= 1;
              MaxValue:= 255;
              OnChange:= SpinEditChange;
            end;
            II:= II + 1;
          end;
          tpTColor: begin
            SetLength(Items,Length(Items)+1);
            Items[II]:= TShape.Create(Self);
            with Items[II] as TShape do begin
              Parent:= Self;
              Top:= ItemTop;
              ItemTop:= ItemTop + 36;
              Left:= 0;
              Width:= 33;
              Height:= 33;
              Brush.Color:= TColorType(Properties[I].Data).Color;
              OnMouseDown:= ShapeMouseDown;
            end;
            II:= II + 1;
          end;
          tpBoolean: begin
            SetLength(Items,Length(Items)+1);
            Items[II]:= TCheckBox.Create(Self);
            with Items[II] as TCheckBox do begin
              Parent:= Self;
              Top:= ItemTop;
              ItemTop:= ItemTop + 20;
              Left:= 0;
              Checked:= TBooleanType(Properties[I].Data).Value;
              Caption:= 'Zapnuto';
              OnClick:= CheckBoxClick;
            end;
            II:= II + 1;
          end;
          tpReal: begin
            SetLength(Items,Length(Items)+1);
            Items[II]:= TMemo.Create(Self);
            with Items[II] as TMemo do begin
              Parent:= Self;
              Top:= ItemTop;
              ItemTop:= ItemTop + 22;
              Left:= 0;
              Width:= 60;
              Height:= 20;
              Lines.Strings[0]:= FloatToStr(TRealType(Properties[I].Data).Value);
              OnChange:= TrackBarChange;
            end;
            II:= II + 1;
          end;
        end;
        Items[II]:= TBevel.Create(Self);
        with Items[II] as TBevel do begin
          Parent:= Self;
          Top:= ItemTop;
          Left:= 0;
          Width:= PropertiesForm.Width-10;
          Height:= 3;
          ItemTop:= ItemTop + 5;
        end;
        II:= II + 1;
      end;
      PropertiesForm.Height:= 30 + ItemTop;
    end;
  InitializingList:= False;
end;

procedure TPropertiesForm.FormCreate(Sender: TObject);
begin
  if MainWindow.CheckBox4.Checked then Show else Hide;
end;

procedure TPropertiesForm.ReReadData;
var
  I, II: Integer;
  RealNum: Extended;
begin
  if not InitializingList then
  with MainWindow do begin
    UpdatingList:= True;
    if Assigned(ListView1.Selected) and Assigned(ListView2.Selected) and
    (ListView1.Selected.SubItems.Strings[0]<>'Celkem') then begin
      VideoEnabled:= False;
      while Zamek do Application.ProcessMessages;

      II:= 0;
      with Methods[ListView2.Selected.Index].Steps[ListView1.Selected.Index] do
      for I:= 0 to High(Properties) do begin
        II:= II + 1;
        if Assigned(Items[II]) then
        case Properties[I].Typ of
          tpInteger: begin
            TIntegerType(Properties[I].Data).Value:= TTrackBar(Items[II]).Position;
            II:= II + 1;
          end;
          tpTColor: TColorType(Properties[I].Data).Color:= TShape(Items[II]).Brush.Color;
          tpBoolean: TBooleanType(Properties[I].Data).Value:= TCheckBox(Items[II]).Checked;
          tpReal: if TryStrToFloat(TMemo(Items[II]).Lines.Strings[0],RealNum) then
            TRealType(Properties[I].Data).Value:= RealNum;
        end;
        II:= II + 2;
      end;
      VideoEnabled:= True;
      Preview.FormPaint(Self);
    end;
    UpdatingList:= False;
  end;
end;

procedure TPropertiesForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  MainWindow.CheckBox4.Checked:= False;
end;

procedure TPropertiesForm.ShapeMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  ColorDialog1.Color:= TShape(Sender).Brush.Color;
  if ColorDialog1.Execute then begin
    TShape(Sender).Brush.Color:= ColorDialog1.Color;
    PropertiesForm.ReReadData;
  end;
end;

procedure TPropertiesForm.TrackBarChange(Sender: TObject);
begin
  //SpinEdit1.Value:= TrackBar1.Position;
  PropertiesForm.ReReadData;
end;

procedure TPropertiesForm.SpinEditChange(Sender: TObject);
begin
//  TrackBar1.Position:= SpinEdit1.Value;
end;

procedure TPropertiesForm.CheckBoxClick(Sender: TObject);
begin
  PropertiesForm.ReReadData;
end;

procedure TPropertiesForm.FormShow(Sender: TObject);
begin
  Left:= MainWindow.Left+MainWindow.Width;
  Top:= MainWindow.Top;
end;

end.
